"""
命令执行结果分析模块
"""
from typing import Dict, List, Tuple, Optional
import re
import json
from dataclasses import dataclass

@dataclass
class CommandResult:
    success: bool
    output: str
    error: str
    analysis: Dict
    suggestions: List[str]

class CommandAnalyzer:
    def __init__(self):
        # 常见错误模式及其解决方案
        self.error_patterns = {
            r"'.*' is not recognized as .+": "命令不存在或未安装",
            r"Access is denied": "权限不足",
            r"The system cannot find the path specified": "路径不存在",
            r"The system cannot find the file specified": "文件不存在",
            r"(.+) is not recognized as an internal or external command": "命令未找到"
        }
        
        # 常见命令输出模式
        self.output_patterns = {
            'dir': self._analyze_dir_output,
            'tasklist': self._analyze_tasklist_output,
            'systeminfo': self._analyze_systeminfo_output,
            'ipconfig': self._analyze_ipconfig_output
        }

    def analyze(self, command: str, returncode: int, stdout: str, stderr: str) -> CommandResult:
        """分析命令执行结果"""
        # 基本分析
        success = returncode == 0
        analysis = {
            'command_type': self._get_command_type(command),
            'execution_status': 'success' if success else 'failed',
            'has_output': bool(stdout.strip()),
            'has_error': bool(stderr.strip())
        }
        
        # 错误分析
        if stderr:
            analysis['error_type'] = self._analyze_error(stderr)
        
        # 输出分析
        if stdout:
            cmd = command.split()[0].lower()
            if cmd in self.output_patterns:
                analysis['output_analysis'] = self.output_patterns[cmd](stdout)
        
        # 生成建议
        suggestions = self._generate_suggestions(command, success, analysis)
        
        return CommandResult(
            success=success,
            output=stdout,
            error=stderr,
            analysis=analysis,
            suggestions=suggestions
        )

    def _get_command_type(self, command: str) -> str:
        """确定命令类型"""
        cmd = command.split()[0].lower()
        types = {
            ('dir', 'ls', 'type', 'cat'): 'file_operation',
            ('tasklist', 'ps', 'top'): 'process_management',
            ('ipconfig', 'ifconfig', 'ping'): 'network',
            ('systeminfo', 'uname'): 'system_info'
        }
        
        for commands, cmd_type in types.items():
            if cmd in commands:
                return cmd_type
        return 'unknown'

    def _analyze_error(self, error: str) -> str:
        """分析错误信息"""
        for pattern, error_type in self.error_patterns.items():
            if re.search(pattern, error, re.IGNORECASE):
                return error_type
        return "unknown_error"

    def _analyze_dir_output(self, output: str) -> Dict:
        """分析dir命令输出"""
        analysis = {
            'file_count': 0,
            'dir_count': 0,
            'total_size': 0
        }
        
        for line in output.split('\n'):
            if '<DIR>' in line:
                analysis['dir_count'] += 1
            elif re.search(r'\d+\s+File\(s\)', line):
                match = re.search(r'(\d+)\s+File\(s\)', line)
                if match:
                    analysis['file_count'] = int(match.group(1))
            elif re.search(r'\d+\s+bytes', line):
                match = re.search(r'(\d+)\s+bytes', line)
                if match:
                    analysis['total_size'] = int(match.group(1))
        
        return analysis

    def _analyze_tasklist_output(self, output: str) -> Dict:
        """分析tasklist命令输出"""
        analysis = {
            'process_count': 0,
            'memory_usage': 0,
            'top_processes': []
        }
        
        processes = []
        for line in output.split('\n')[3:]:  # 跳过标题行
            parts = line.split()
            if len(parts) >= 5:
                mem = parts[4].replace(',', '').replace('K', '')
                try:
                    memory = int(mem)
                    processes.append((parts[0], memory))
                    analysis['memory_usage'] += memory
                except ValueError:
                    continue
        
        analysis['process_count'] = len(processes)
        processes.sort(key=lambda x: x[1], reverse=True)
        analysis['top_processes'] = processes[:5]
        
        return analysis

    def _analyze_systeminfo_output(self, output: str) -> Dict:
        """分析systeminfo命令输出"""
        analysis = {}
        key_items = {
            'OS Name': 'os_name',
            'OS Version': 'os_version',
            'System Manufacturer': 'manufacturer',
            'System Model': 'model',
            'Total Physical Memory': 'total_memory'
        }
        
        for line in output.split('\n'):
            for key, value in key_items.items():
                if line.startswith(key):
                    analysis[value] = line.split(':')[-1].strip()
        
        return analysis

    def _analyze_ipconfig_output(self, output: str) -> Dict:
        """分析ipconfig命令输出"""
        analysis = {
            'adapters': [],
            'active_adapters': 0
        }
        
        current_adapter = None
        for line in output.split('\n'):
            if not line.startswith(' ') and ':' in line:
                current_adapter = {
                    'name': line.split(':')[0].strip(),
                    'ip_addresses': []
                }
                analysis['adapters'].append(current_adapter)
            elif current_adapter and 'IPv4' in line:
                ip = re.search(r'\d+\.\d+\.\d+\.\d+', line)
                if ip:
                    current_adapter['ip_addresses'].append(ip.group())
                    analysis['active_adapters'] += 1
        
        return analysis

    def _generate_suggestions(self, command: str, success: bool, analysis: Dict) -> List[str]:
        """生成建议"""
        suggestions = []
        
        if not success:
            error_type = analysis.get('error_type', '')
            if 'command not found' in error_type:
                suggestions.append(f"请检查命令 '{command.split()[0]}' 是否安装或是否在系统PATH中")
            elif 'permission denied' in error_type:
                suggestions.append("尝试使用管理员权限运行此命令")
        else:
            cmd_type = analysis['command_type']
            if cmd_type == 'process_management':
                suggestions.append("使用 'taskkill /PID pid' 来终止特定进程")
            elif cmd_type == 'network':
                suggestions.append("使用 'ipconfig /release' 和 'ipconfig /renew' 来刷新网络配置")
        
        return suggestions
