"""
上下文管理模块
"""
from typing import List, Dict, Optional
import json
import os
from datetime import datetime

class ContextManager:
    def __init__(self, max_history: int = 10):
        self.max_history = max_history
        self.conversation_history: List[Dict] = []
        self.current_directory: str = os.getcwd()
        self.last_command: Optional[str] = None
        self.last_command_status: Optional[bool] = None
        self.environment_vars: Dict[str, str] = dict(os.environ)

    def add_interaction(self, user_input: str, assistant_response: str) -> None:
        """添加新的交互记录"""
        self.conversation_history.append({
            'timestamp': datetime.now().isoformat(),
            'user_input': user_input,
            'assistant_response': assistant_response,
            'current_directory': self.current_directory,
            'last_command': self.last_command,
            'last_command_status': self.last_command_status
        })
        
        # 保持历史记录在限定大小内
        if len(self.conversation_history) > self.max_history:
            self.conversation_history.pop(0)

    def update_command_context(self, command: str, status: bool) -> None:
        """更新命令执行上下文"""
        self.last_command = command
        self.last_command_status = status

    def update_directory_context(self, directory: str) -> None:
        """更新目录上下文"""
        self.current_directory = directory

    def get_context_for_prompt(self) -> str:
        """获取用于提示的上下文信息"""
        context = [
            f"当前目录: {self.current_directory}",
        ]
        
        if self.last_command:
            status = "成功" if self.last_command_status else "失败"
            context.append(f"上一条命令: {self.last_command} ({status})")
        
        if self.conversation_history:
            last_interaction = self.conversation_history[-1]
            context.append(f"上一次交互: {last_interaction['user_input']}")
        
        return "\n".join(context)

    def get_relevant_history(self, query: str, limit: int = 3) -> List[Dict]:
        """获取与当前查询相关的历史记录"""
        # 简单的相关性匹配，实际项目中可以使用更复杂的算法
        relevant = []
        for interaction in reversed(self.conversation_history):
            if any(word in interaction['user_input'].lower() 
                  for word in query.lower().split()):
                relevant.append(interaction)
                if len(relevant) >= limit:
                    break
        return relevant

    def save_context(self, filename: str) -> None:
        """保存上下文到文件"""
        with open(filename, 'w', encoding='utf-8') as f:
            json.dump({
                'conversation_history': self.conversation_history,
                'current_directory': self.current_directory,
                'last_command': self.last_command,
                'last_command_status': self.last_command_status
            }, f, ensure_ascii=False, indent=2)

    def load_context(self, filename: str) -> None:
        """从文件加载上下文"""
        try:
            with open(filename, 'r', encoding='utf-8') as f:
                data = json.load(f)
                self.conversation_history = data['conversation_history']
                self.current_directory = data['current_directory']
                self.last_command = data['last_command']
                self.last_command_status = data['last_command_status']
        except FileNotFoundError:
            pass
