"""
工具函数模块，提供通用功能
"""
import platform
import subprocess
from typing import List, Optional
import json

def get_system_info() -> str:
    """获取系统信息"""
    return platform.system()

def execute_command(command: str) -> tuple[int, str, str]:
    """
    执行系统命令
    返回: (返回码, 标准输出, 标准错误)
    """
    try:
        process = subprocess.Popen(
            command,
            shell=True,
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
            text=True
        )
        stdout, stderr = process.communicate()
        return process.returncode, stdout, stderr
    except Exception as e:
        return 1, "", str(e)

def is_safe_command(command: str) -> bool:
    """检查命令是否安全"""
    dangerous_commands = [
        "rm -rf", "format", "del /f", "deltree",
        ">", ">>", "|", ";", "&", "&&", "||"
    ]
    return not any(cmd in command.lower() for cmd in dangerous_commands)

class CommandHistory:
    def __init__(self, max_size: int = 100):
        self.history: List[str] = []
        self.max_size = max_size

    def add(self, command: str) -> None:
        """添加命令到历史记录"""
        self.history.append(command)
        if len(self.history) > self.max_size:
            self.history.pop(0)

    def get_last(self, n: int = 1) -> List[str]:
        """获取最近的n条命令"""
        return self.history[-n:]

    def clear(self) -> None:
        """清空历史记录"""
        self.history.clear()

    def save_to_file(self, filename: str) -> None:
        """保存历史记录到文件"""
        with open(filename, 'w') as f:
            json.dump(self.history, f)

    def load_from_file(self, filename: str) -> None:
        """从文件加载历史记录"""
        try:
            with open(filename, 'r') as f:
                self.history = json.load(f)
        except FileNotFoundError:
            self.history = []
